'use strict';

const chai = require('chai');
const expect = chai.expect;
chai.use(require('chai-as-promised'));
// const sinon = require('sinon');
// chai.use(require('sinon-chai'));
const mockery = require('mockery');
const redis = require('redis-mock');
const Etcd = require('node-etcd');
const promisify = require('es6-promisify');
const co = require('co');

let Ravel, app, etcd;

describe('Ravel MySQLProvider', () => {
  beforeEach(done => {
    process.removeAllListeners('unhandledRejection');
    //enable mockery
    mockery.enable({
      useCleanCache: true,
      warnOnReplace: false,
      warnOnUnregistered: false });

    mockery.registerMock('redis', redis);
    Ravel = require('ravel');
    app = new Ravel();
    // app.log.setLevel(app.log.NONE);
    app.set('log level', app.log.NONE);
    app.set('keygrip keys', ['mysecret']);

    const client = new Etcd('127.0.0.1:2379');
    etcd = {
      set: promisify(client.set, client),
      get: promisify(client.get, client),
      del: promisify(client.del, client),
      mkdir: promisify(client.mkdir, client),
      rmdir: promisify(client.rmdir, client) };


    done();
  });

  afterEach(done => {
    process.removeAllListeners('unhandledRejection');
    mockery.deregisterAll();
    mockery.disable();
    app.close();
    done();
  });

  describe('post load parameters', () => {
    it('should load configuration parameters from etcd', () => {
      return co(function* () {
        try {
          yield etcd.rmdir('/ravelrc/', { recursive: true });
        } catch (err) {} /* do nothing */ //eslint-disable-line no-empty
        yield etcd.mkdir('/ravelrc/');
        yield etcd.set('/ravelrc/redis host', '1.2.3.4');
        const EtcdConfig = require('../lib/ravel-etcd-config');
        const conf = new EtcdConfig(app); //eslint-disable-line no-unused-vars
        app.init();
        expect(app.get('redis host')).to.equal('1.2.3.4');
        yield etcd.rmdir('/ravelrc/', { recursive: true });
      });
    });

    it('should support customized folder paths', () => {
      return co(function* () {
        try {
          yield etcd.rmdir('/myconfig/', { recursive: true });
        } catch (err) {} /* do nothing */ //eslint-disable-line no-empty
        yield etcd.mkdir('/myconfig/');
        yield etcd.set('/myconfig/redis port', '1234');
        const EtcdConfig = require('../lib/ravel-etcd-config');
        const conf = new EtcdConfig(app, 'myconfig'); //eslint-disable-line no-unused-vars
        app.init();
        expect(app.get('redis port')).to.equal(1234);
        yield etcd.rmdir('/myconfig/', { recursive: true });
      });
    });

    it('should support object values', () => {
      return co(function* () {
        const obj = { a: '1', b: '2' };
        try {
          yield etcd.rmdir('/ravelrc/', { recursive: true });
        } catch (err) {} /* do nothing */ //eslint-disable-line no-empty
        yield etcd.mkdir('/ravelrc/');
        yield etcd.set('/ravelrc/redis host', JSON.stringify(obj));
        const EtcdConfig = require('../lib/ravel-etcd-config');
        const conf = new EtcdConfig(app); //eslint-disable-line no-unused-vars
        app.init();
        expect(app.get('redis host')).to.deep.equal(obj);
        yield etcd.rmdir('/ravelrc/', { recursive: true });
      });
    });

    it('should handle empty responses from etcd', () => {
      const mock = function () {
        return {
          getSync: () => {return {};} };

      };
      mockery.registerMock('node-etcd', mock);
      const EtcdConfig = require('../lib/ravel-etcd-config');
      const conf = new EtcdConfig(app); //eslint-disable-line no-unused-vars
      return expect(() => app.init()).to.throw(Error);
    });

    it('should handle malformed responses from etcd (incorrect body)', () => {
      const mock = function () {
        return {
          getSync: () => {return { body: 'a string' };} };

      };
      mockery.registerMock('node-etcd', mock);
      const EtcdConfig = require('../lib/ravel-etcd-config');
      const conf = new EtcdConfig(app); //eslint-disable-line no-unused-vars
      return expect(() => app.init()).to.throw(Error);
    });

    it('should handle malformed responses from etcd (missing nodes)', () => {
      const mock = function () {
        return {
          getSync: () => {return { body: {} };} };

      };
      mockery.registerMock('node-etcd', mock);
      const EtcdConfig = require('../lib/ravel-etcd-config');
      const conf = new EtcdConfig(app); //eslint-disable-line no-unused-vars
      return expect(() => app.init()).to.throw(Error);
    });

    it('should handle malformed responses from etcd (missing nodes)', () => {
      const mock = function () {
        return {
          getSync: () => {return { body: { node: {} } };} };

      };
      mockery.registerMock('node-etcd', mock);
      const EtcdConfig = require('../lib/ravel-etcd-config');
      const conf = new EtcdConfig(app); //eslint-disable-line no-unused-vars
      return expect(() => app.init()).to.throw(Error);
    });

    it('should throw an error when it can not connect to etcd', () => {
      const EtcdConfig = require('../lib/ravel-etcd-config');
      const conf = new EtcdConfig(app); //eslint-disable-line no-unused-vars
      app.set('config etcd host', '127.0.0.1:20379');
      expect(() => app.init()).to.throw(Error);
    });
  });
});
//# sourceMappingURL=test-ravel-etcd-config.js.map
